/**
 * Managing Large Data, Systems Group, ETH Zurich
 * Exercise 7
 * Bitmap Indexing, Query Processing and WAH Compression
 *
 * File        : BitmapIndex.h
 * Author      : Cagri Balkesen, ETH Zurich, 2012-14-05
 *
 */

#ifndef _BITMAP_INDEX_H_
#define _BITMAP_INDEX_H_

#include <stdint.h>

#include "schema.h"
#include "FieldType.h"
#include "BitVector.h"

template <typename T>
class BitmapIndex {
  private:
    FieldType<T> mFieldType;
    uint32_t mNumrows;
    BitVector ** mBitVectors;

  public:
    BitmapIndex(FieldType<T> & fieldType, uint32_t numrows) 
        : mFieldType(fieldType), mNumrows(numrows)
      {
          mBitVectors = new BitVector*[mFieldType.getFieldCardinality()];
          for(uint32_t i = 0; i < mFieldType.getFieldCardinality(); i++) {
              mBitVectors[i] = new BitVector(mNumrows);
          }
      }

    ~BitmapIndex()
      {
          for(uint32_t i = 0; i < mFieldType.getFieldCardinality(); i++) {
              delete mBitVectors[i];
          }

          delete [] mBitVectors;
      }

    uint32_t buildIndex(lineitem_t * inputtable) {
        for(uint32_t i = 0; i < mNumrows; i++) {
            int32_t vectIdx = mFieldType.lookup(&inputtable[i]);

            // Set the bit at the given bit-vector (vectIdx) at index i
            mBitVectors[vectIdx]->nextSetBitIndex(i);
        }

        uint32_t nWords = 0;
        // finally compute number of words used for the index
        for(uint32_t i = 0; i < mFieldType.getFieldCardinality(); i++) {
            nWords += mBitVectors[i]->getNumWords();
        }

        return nWords;
    }

    const BitVector & getBitVector(const T & fieldValue) {
        int32_t vectIdx = mFieldType.lookupByValue(fieldValue);

        return *mBitVectors[vectIdx];
    }
};

#endif /* _BITMAP_INDEX_H_ */
