/**
 * Managing Large Data, Systems Group, ETH Zurich
 * Exercise 7
 * Bitmap Indexing, Query Processing and WAH Compression
 *
 * File        : WAHBitVector.cpp
 * Author      : Cagri Balkesen, ETH Zurich, 2012-14-05
 *
 */

#include "WAHBitVector.h"

/***** Global bit operations on WAHBitVectors *****/

/* AND operator with a temporary result */
WAHBitVector
operator& (const WAHBitVector& lhs, const WAHBitVector& rhs) 
{
    /* Assumption: vector lengths will be same. */
    uint32_t x, y;
    WAHBitVector result(lhs.size());
    WAHWordIterator lhsItr(lhs.mBitWords, 
                           lhs.mActiveWord.to_ulong(), lhs.mActiveWordSize);
    WAHWordIterator rhsItr(rhs.mBitWords, 
                           rhs.mActiveWord.to_ulong(), rhs.mActiveWordSize);

    lhsItr.begin();
    rhsItr.begin();

    while( lhsItr.hasMoreWords() && rhsItr.hasMoreWords() ) {
        x = lhsItr.getNextDecodedWord();
        y = rhsItr.getNextDecodedWord();

        result.appendUncompressed(x & y);
    }

    while( lhsItr.hasMoreWords() ) {
        // all remaining y's are 0
        x = lhsItr.getNextDecodedWord();
        result.appendUncompressed(0);// x & 0 = 0
    }

    while( rhsItr.hasMoreWords() ) {
        // all remaining x's are 0
        y = rhsItr.getNextDecodedWord();
        result.appendUncompressed(0);// y & 0 = 0
    }

    result.mActiveWordSize = 0;

    return result;
}

/* OR operator of two WAHBitVectors in a temporary result */
WAHBitVector 
operator| (const WAHBitVector& lhs, const WAHBitVector& rhs)
{
    /* Assumption: vector lengths will be same. */
    uint32_t x, y;
    WAHBitVector result(rhs.size());
    WAHWordIterator lhsItr(lhs.mBitWords, 
                           lhs.mActiveWord.to_ulong(), lhs.mActiveWordSize);
    WAHWordIterator rhsItr(rhs.mBitWords, 
                           rhs.mActiveWord.to_ulong(), rhs.mActiveWordSize);
    lhsItr.begin();
    rhsItr.begin();

    while( lhsItr.hasMoreWords() && rhsItr.hasMoreWords() ) {
        x = lhsItr.getNextDecodedWord();
        y = rhsItr.getNextDecodedWord();
        result.appendUncompressed(x | y);
    }
    result.mActiveWordSize = 0;

    while( lhsItr.hasMoreWords() ) {
        // all remaining y's are 0
        x = lhsItr.getNextDecodedWord();
        result.appendUncompressed(x);// x | 0 = x
    }

    while( rhsItr.hasMoreWords() ) {
        // all remaining x's are 0
        y = rhsItr.getNextDecodedWord();
        result.appendUncompressed(y);// y | 0 = y
    }

    return result;
}
